@@ -1,6 +1,6 @@
 /*
     instrument-js.cpp - JavaScript instrumentation routines
-    Copyright (C) 2007, 2008 siliconforks.com
+    Copyright (C) 2007, 2008, 2009, 2010 siliconforks.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,6 +26,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+// needed by jsparse.h
+#include <jsbit.h>
+#include <jscntxt.h>
+#include <jsscript.h>
+
 #include <jsapi.h>
 #include <jsarena.h>
 #include <jsatom.h>
@@ -48,12 +53,12 @@
 struct IfDirective {
   const jschar * condition_start;
   const jschar * condition_end;
-  uint16_t start_line;
-  uint16_t end_line;
+  uint32_t start_line;
+  uint32_t end_line;
   struct IfDirective * next;
 };
 
-bool jscoverage_mozilla = false;
+enum JSCoverageMode jscoverage_mode = JSCOVERAGE_NORMAL;
 
 static bool * exclusive_directives = NULL;
 
@@ -68,7 +73,7 @@ The lines array stores line numbers starting from 0.
 */
 static const char * file_id = NULL;
 static char * lines = NULL;
-static uint16_t num_lines = 0;
+static uint32_t num_lines = 0;
 
 void jscoverage_set_js_version(const char * version) {
   js_version = JS_StringToVersion(version);
@@ -136,8 +141,9 @@ static void print_javascript(const jschar * characters, size_t num_characters, S
 }
 
 static void print_string(JSString * s, Stream * f) {
-  size_t length = JSSTRING_LENGTH(s);
-  jschar * characters = JSSTRING_CHARS(s);
+  size_t length;
+  const jschar * characters;
+  s->getCharsAndLength(characters, length);
   for (size_t i = 0; i < length; i++) {
     jschar c = characters[i];
     if (32 <= c && c <= 126) {
@@ -198,8 +204,9 @@ static void print_string_atom(JSAtom * atom, Stream * f) {
 static void print_regex(jsval value, Stream * f) {
   assert(JSVAL_IS_STRING(value));
   JSString * s = JSVAL_TO_STRING(value);
-  size_t length = JSSTRING_LENGTH(s);
-  jschar * characters = JSSTRING_CHARS(s);
+  size_t length;
+  const jschar * characters;
+  s->getCharsAndLength(characters, length);
   for (size_t i = 0; i < length; i++) {
     jschar c = characters[i];
     if (32 <= c && c <= 126) {
@@ -268,7 +275,7 @@ static const char * get_op(uint8 op) {
   }
 }
 
-static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_object_literals);
+static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_object_literals, bool parenthesize_assignments = true);
 static void instrument_statement(JSParseNode * node, Stream * f, int indent, bool is_jscoverage_if);
 static void output_statement(JSParseNode * node, Stream * f, int indent, bool is_jscoverage_if);
 
@@ -284,9 +291,7 @@ static void output_for_in(JSParseNode * node, Stream * f) {
   if (node->pn_iflags & JSITER_FOREACH) {
     Stream_write_string(f, "each ");
   }
-  Stream_write_char(f, '(');
   output_expression(node->pn_left, f, false);
-  Stream_write_char(f, ')');
 }
 
 static void output_array_comprehension_or_generator_expression(JSParseNode * node, Stream * f) {
@@ -349,7 +354,7 @@ static void output_array_comprehension_or_generator_expression(JSParseNode * nod
 static void instrument_function(JSParseNode * node, Stream * f, int indent, enum FunctionType type) {
   assert(node->pn_type == TOK_FUNCTION);
   assert(node->pn_arity == PN_FUNC);
-  JSObject * object = node->pn_funpob->object;
+  JSObject * object = node->pn_funbox->object;
   assert(JS_ObjectIsFunction(context, object));
   JSFunction * function = (JSFunction *) JS_GetPrivate(context, object);
   assert(function);
@@ -372,7 +377,7 @@ static void instrument_function(JSParseNode * node, Stream * f, int indent, enum
   JSArenaPool pool;
   JS_INIT_ARENA_POOL(&pool, "instrument_function", 256, 1, &context->scriptStackQuota);
   jsuword * local_names = NULL;
-  if (JS_GET_LOCAL_NAME_COUNT(function)) {
+  if (function->hasLocalNames()) {
     local_names = js_GetLocalNameArray(context, function, &pool);
     if (local_names == NULL) {
       fatal("out of memory");
@@ -386,17 +391,33 @@ static void instrument_function(JSParseNode * node, Stream * f, int indent, enum
     JSAtom * param = JS_LOCAL_NAME_TO_ATOM(local_names[i]);
     if (param == NULL) {
       destructuring = true;
+
+      JSParseNode * p = node->pn_body;
+      if (p->pn_type == TOK_UPVARS) {
+        assert(p->pn_arity == PN_NAMESET);
+        p = p->pn_tree;
+      }
+
+      if (p->pn_type == TOK_ARGSBODY) {
+        assert(p->pn_arity == PN_LIST);
+        // the function body is the final element of the list
+        for (p = p->pn_head; p->pn_next != NULL; p = p->pn_next) {
+          ;
+        }
+      }
+
       JSParseNode * expression = NULL;
-      assert(node->pn_body->pn_type == TOK_LC || node->pn_body->pn_type == TOK_SEQ);
-      JSParseNode * semi = node->pn_body->pn_head;
+      assert(p->pn_type == TOK_LC || p->pn_type == TOK_SEQ);
+      assert(p->pn_arity == PN_LIST);
+      JSParseNode * semi = p->pn_head;
       assert(semi->pn_type == TOK_SEMI);
       JSParseNode * comma = semi->pn_kid;
       assert(comma->pn_type == TOK_COMMA);
       for (JSParseNode * p = comma->pn_head; p != NULL; p = p->pn_next) {
         assert(p->pn_type == TOK_ASSIGN);
         JSParseNode * rhs = p->pn_right;
-        assert(JSSTRING_LENGTH(ATOM_TO_STRING(rhs->pn_atom)) == 0);
-        if (rhs->pn_slot == i) {
+        assert(ATOM_TO_STRING(rhs->pn_atom)->length() == 0);
+        if (UPVAR_FRAME_SLOT(rhs->pn_cookie) == i) {
           expression = p->pn_left;
           break;
         }
@@ -412,21 +433,36 @@ static void instrument_function(JSParseNode * node, Stream * f, int indent, enum
   Stream_write_string(f, ") {\n");
 
   /* function body */
+
+  JSParseNode * p = node->pn_body;
+  if (p->pn_type == TOK_UPVARS) {
+    assert(p->pn_arity == PN_NAMESET);
+    p = p->pn_tree;
+  }
+
+  if (p->pn_type == TOK_ARGSBODY) {
+    assert(p->pn_arity == PN_LIST);
+    // the function body is the final element of the list
+    for (p = p->pn_head; p->pn_next != NULL; p = p->pn_next) {
+      ;
+    }
+  }
+
   if (function->flags & JSFUN_EXPR_CLOSURE) {
     /* expression closure - use output_statement instead of instrument_statement */
-    if (node->pn_body->pn_type == TOK_SEQ) {
-      assert(node->pn_body->pn_arity == PN_LIST);
-      assert(node->pn_body->pn_count == 2);
-      output_statement(node->pn_body->pn_head->pn_next, f, indent + 2, false);
+    if (p->pn_type == TOK_SEQ) {
+      assert(p->pn_arity == PN_LIST);
+      assert(p->pn_count == 2);
+      output_statement(p->pn_head->pn_next, f, indent + 2, false);
     }
     else {
-      output_statement(node->pn_body, f, indent + 2, false);
+      output_statement(p, f, indent + 2, false);
     }
   }
   else {
-    assert(node->pn_body->pn_type == TOK_LC);
-    assert(node->pn_body->pn_arity == PN_LIST);
-    JSParseNode * p = node->pn_body->pn_head;
+    assert(p->pn_type == TOK_LC);
+    assert(p->pn_arity == PN_LIST); 
+    p = p->pn_head;
     if (destructuring) {
       p = p->pn_next;
     }
@@ -438,27 +474,11 @@ static void instrument_function(JSParseNode * node, Stream * f, int indent, enum
   Stream_write_char(f, '}');
 }
 
-static void instrument_function_call(JSParseNode * node, Stream * f) {
+static void output_function_arguments(JSParseNode * node, Stream * f) {
   JSParseNode * function_node = node->pn_head;
-  if (function_node->pn_type == TOK_FUNCTION) {
-    JSObject * object = function_node->pn_funpob->object;
-    assert(JS_ObjectIsFunction(context, object));
-    JSFunction * function = (JSFunction *) JS_GetPrivate(context, object);
-    assert(function);
-    assert(object == &function->object);
-
-    if (function_node->pn_flags & TCF_GENEXP_LAMBDA) {
-      /* it's a generator expression */
-      Stream_write_char(f, '(');
-      output_array_comprehension_or_generator_expression(function_node->pn_body, f);
-      Stream_write_char(f, ')');
-      return;
-    }
-  }
-  output_expression(function_node, f, false);
   Stream_write_char(f, '(');
   for (struct JSParseNode * p = function_node->pn_next; p != NULL; p = p->pn_next) {
-    if (p != node->pn_head->pn_next) {
+    if (p != function_node->pn_next) {
       Stream_write_string(f, ", ");
     }
     output_expression(p, f, false);
@@ -466,13 +486,68 @@ static void instrument_function_call(JSParseNode * node, Stream * f) {
   Stream_write_char(f, ')');
 }
 
+static void instrument_function_call(JSParseNode * node, Stream * f) {
+  JSParseNode * function_node = node->pn_head;
+  if (function_node->pn_type == TOK_FUNCTION) {
+    JSObject * object = function_node->pn_funbox->object;
+    assert(JS_ObjectIsFunction(context, object));
+    JSFunction * function = (JSFunction *) JS_GetPrivate(context, object);
+    assert(function);
+    assert(object == &function->object);
+
+    if (function_node->pn_funbox->tcflags & TCF_GENEXP_LAMBDA) {
+      /* it's a generator expression */
+      Stream_write_char(f, '(');
+      JSParseNode * p = function_node->pn_body;
+      if (p->pn_type == TOK_UPVARS) {
+        assert(p->pn_arity == PN_NAMESET);
+        p = p->pn_tree;
+      }
+      output_array_comprehension_or_generator_expression(p, f);
+      Stream_write_char(f, ')');
+      return;
+    }
+  }
+
+  // put parentheses around anything that isn't a name or dot
+  switch (function_node->pn_type) {
+  case TOK_NAME:
+  case TOK_DOT:
+
+  // FIXME: TOK_FUNCTION is here because parentheses are already added in output_expression
+  case TOK_FUNCTION:
+
+    output_expression(function_node, f, false);
+    break;
+  default:
+    Stream_write_char(f, '(');
+    output_expression(function_node, f, false);
+    Stream_write_char(f, ')');
+    break;
+  }
+
+  output_function_arguments(node, f);
+}
+
 static void instrument_declarations(JSParseNode * list, Stream * f) {
   assert(list->pn_arity == PN_LIST);
   for (JSParseNode * p = list->pn_head; p != NULL; p = p->pn_next) {
     if (p != list->pn_head) {
       Stream_write_string(f, ", ");
     }
-    output_expression(p, f, false);
+
+    switch (p->pn_type) {
+    case TOK_NAME:
+      print_string_atom(p->pn_atom, f);
+      if (p->pn_expr != NULL) {
+        Stream_write_string(f, " = ");
+        output_expression(p->pn_expr, f, false);
+      }
+      break;
+    default:
+      output_expression(p, f, false, false);
+      break;
+    }
   }
 }
 
@@ -489,7 +564,7 @@ There seem to be some undocumented expressions:
 TOK_INSTANCEOF  binary
 TOK_IN          binary
 */
-static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_object_literals) {
+static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_object_literals, bool parenthesize_assignments) {
   switch (node->pn_type) {
   case TOK_FUNCTION:
     Stream_write_char(f, '(');
@@ -497,14 +572,19 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     Stream_write_char(f, ')');
     break;
   case TOK_COMMA:
+    Stream_write_char(f, '(');
     for (struct JSParseNode * p = node->pn_head; p != NULL; p = p->pn_next) {
       if (p != node->pn_head) {
         Stream_write_string(f, ", ");
       }
       output_expression(p, f, parenthesize_object_literals);
     }
+    Stream_write_char(f, ')');
     break;
   case TOK_ASSIGN:
+    if (parenthesize_assignments) {
+      Stream_write_char(f, '(');
+    }
     output_expression(node->pn_left, f, parenthesize_object_literals);
     Stream_write_char(f, ' ');
     switch (node->pn_op) {
@@ -527,13 +607,18 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     }
     Stream_write_string(f, "= ");
     output_expression(node->pn_right, f, false);
+    if (parenthesize_assignments) {
+      Stream_write_char(f, ')');
+    }
     break;
   case TOK_HOOK:
+    Stream_write_char(f, '(');
     output_expression(node->pn_kid1, f, parenthesize_object_literals);
     Stream_write_string(f, "? ");
     output_expression(node->pn_kid2, f, false);
     Stream_write_string(f, ": ");
     output_expression(node->pn_kid3, f, false);
+    Stream_write_char(f, ')');
     break;
   case TOK_OR:
   case TOK_AND:
@@ -547,6 +632,7 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
   case TOK_MINUS:
   case TOK_STAR:
   case TOK_DIVOP:
+    Stream_write_char(f, '(');
     switch (node->pn_arity) {
     case PN_BINARY:
       output_expression(node->pn_left, f, parenthesize_object_literals);
@@ -567,8 +653,10 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     default:
       abort();
     }
+    Stream_write_char(f, ')');
     break;
   case TOK_UNARYOP:
+    Stream_write_char(f, '(');
     switch (node->pn_op) {
     case JSOP_NEG:
       Stream_write_string(f, "- ");
@@ -587,6 +675,7 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
       output_expression(node->pn_kid, f, false);
       break;
     case JSOP_TYPEOF:
+    case JSOP_TYPEOFEXPR:
       Stream_write_string(f, "typeof ");
       output_expression(node->pn_kid, f, false);
       break;
@@ -595,15 +684,17 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
       output_expression(node->pn_kid, f, false);
       break;
     default:
-      fatal_source(file_id, node->pn_pos.begin.lineno, "unknown operator (%d)", node->pn_op);
+      fatal_source(file_id, node->pn_pos.begin.lineno, "unknown operator (%u)", (unsigned int) node->pn_op);
       break;
     }
+    Stream_write_char(f, ')');
     break;
   case TOK_INC:
   case TOK_DEC:
     /*
     This is not documented, but node->pn_op tells whether it is pre- or post-increment.
     */
+    Stream_write_char(f, '(');
     switch (node->pn_op) {
     case JSOP_INCNAME:
     case JSOP_INCPROP:
@@ -633,14 +724,34 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
       abort();
       break;
     }
+    Stream_write_char(f, ')');
     break;
   case TOK_NEW:
+    /*
+    For an expression like
+      new (f())();
+    SpiderMonkey creates a node with pn_head NOT in parentheses.  If we just
+    output pn_head, we end up with
+      new f()();
+    This is not correct, because it is parsed as
+      (new f())();
+    We can fix this by surrounding pn_head in parentheses.
+    */
     Stream_write_string(f, "new ");
-    instrument_function_call(node, f);
+    if (node->pn_head->pn_type != TOK_NAME) {
+      Stream_write_char(f, '(');
+    }
+    output_expression(node->pn_head, f, false);
+    if (node->pn_head->pn_type != TOK_NAME) {
+      Stream_write_char(f, ')');
+    }
+    output_function_arguments(node, f);
     break;
   case TOK_DELETE:
+    Stream_write_char(f, '(');
     Stream_write_string(f, "delete ");
     output_expression(node->pn_kid, f, false);
+    Stream_write_char(f, ')');
     break;
   case TOK_DOT:
     /* numeric literals must be parenthesized */
@@ -663,10 +774,15 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     {
       JSString * s = ATOM_TO_STRING(node->pn_atom);
       bool must_quote;
-      if (JSSTRING_LENGTH(s) == 0) {
+
+      size_t length;
+      const jschar * characters;
+      s->getCharsAndLength(characters, length);
+
+      if (length == 0) {
         must_quote = true;
       }
-      else if (js_CheckKeyword(JSSTRING_CHARS(s), JSSTRING_LENGTH(s)) != TOK_EOF) {
+      else if (js_CheckKeyword(characters, length) != TOK_EOF) {
         must_quote = true;
       }
       else if (! js_IsIdentifier(s)) {
@@ -701,12 +817,12 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
       if (p != node->pn_head) {
         Stream_write_string(f, ", ");
       }
-      /* TOK_COMMA is a special case: a hole in the array */
-      if (p->pn_type != TOK_COMMA) {
+      /* a TOK_COMMA which is not a PN_LIST is a special case: a hole in the array */
+      if (! (p->pn_type == TOK_COMMA && p->pn_arity != PN_LIST)) {
         output_expression(p, f, false);
       }
     }
-    if (node->pn_extra == PNX_ENDCOMMA) {
+    if (node->pn_xflags & PNX_ENDCOMMA) {
       Stream_write_char(f, ',');
     }
     Stream_write_char(f, ']');
@@ -718,7 +834,7 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     Stream_write_char(f, '{');
     for (struct JSParseNode * p = node->pn_head; p != NULL; p = p->pn_next) {
       if (p->pn_type != TOK_COLON) {
-        fatal_source(file_id, p->pn_pos.begin.lineno, "unsupported node type (%d)", p->pn_type);
+        fatal_source(file_id, p->pn_pos.begin.lineno, "unsupported node type (%u)", (unsigned int) p->pn_type);
       }
       if (p != node->pn_head) {
         Stream_write_string(f, ", ");
@@ -760,10 +876,6 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     break;
   case TOK_NAME:
     print_string_atom(node->pn_atom, f);
-    if (node->pn_expr != NULL) {
-      Stream_write_string(f, " = ");
-      output_expression(node->pn_expr, f, false);
-    }
     break;
   case TOK_STRING:
     print_quoted_string_atom(node->pn_atom, f);
@@ -771,7 +883,7 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
   case TOK_REGEXP:
     assert(node->pn_op == JSOP_REGEXP);
     {
-      JSObject * object = node->pn_pob->object;
+      JSObject * object = node->pn_objbox->object;
       jsval result;
       js_regexp_toString(context, object, &result);
       print_regex(result, f);
@@ -780,8 +892,9 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
   case TOK_NUMBER:
     /*
     A 64-bit IEEE 754 floating point number has a 52-bit fraction.
-    2^(-52) = 2.22 x 10^(-16)
-    Thus there are 16 significant digits.
+    (This represents 53 bits of precision - the first bit is not stored.)
+    17 decimal digits are required to recover the floating-point number.
+    See http://docs.sun.com/source/806-3568/ncg_goldberg.html
     To keep the output simple, special-case zero.
     */
     if (node->pn_dval == 0.0) {
@@ -802,7 +915,7 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
       Stream_write_string(f, "Number.NaN");
     }
     else {
-      Stream_printf(f, "%.15g", node->pn_dval);
+      Stream_printf(f, "%.17g", node->pn_dval);
     }
     break;
   case TOK_PRIMARY:
@@ -825,33 +938,41 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     }
     break;
   case TOK_INSTANCEOF:
+    Stream_write_char(f, '(');
     output_expression(node->pn_left, f, parenthesize_object_literals);
     Stream_write_string(f, " instanceof ");
     output_expression(node->pn_right, f, false);
+    Stream_write_char(f, ')');
     break;
   case TOK_IN:
+    Stream_write_char(f, '(');
     output_expression(node->pn_left, f, false);
     Stream_write_string(f, " in ");
     output_expression(node->pn_right, f, false);
+    Stream_write_char(f, ')');
     break;
   case TOK_LEXICALSCOPE:
     assert(node->pn_arity == PN_NAME);
     assert(node->pn_expr->pn_type == TOK_LET);
     assert(node->pn_expr->pn_arity == PN_BINARY);
+    Stream_write_char(f, '(');
     Stream_write_string(f, "let(");
     assert(node->pn_expr->pn_left->pn_type == TOK_LP);
     assert(node->pn_expr->pn_left->pn_arity == PN_LIST);
     instrument_declarations(node->pn_expr->pn_left, f);
     Stream_write_string(f, ") ");
     output_expression(node->pn_expr->pn_right, f, true);
+    Stream_write_char(f, ')');
     break;
   case TOK_YIELD:
     assert(node->pn_arity == PN_UNARY);
+    Stream_write_char(f, '(');
     Stream_write_string(f, "yield");
     if (node->pn_kid != NULL) {
       Stream_write_char(f, ' ');
       output_expression(node->pn_kid, f, true);
     }
+    Stream_write_char(f, ')');
     break;
   case TOK_ARRAYCOMP:
     assert(node->pn_arity == PN_LIST);
@@ -875,7 +996,12 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     break;
   case TOK_VAR:
     assert(node->pn_arity == PN_LIST);
-    Stream_write_string(f, "var ");
+    if (node->pn_op == JSOP_DEFCONST) {
+      Stream_write_string(f, "const ");
+    }
+    else {
+      Stream_write_string(f, "var ");
+    }
     instrument_declarations(node, f);
     break;
   case TOK_LET:
@@ -884,7 +1010,7 @@ static void output_expression(JSParseNode * node, Stream * f, bool parenthesize_
     instrument_declarations(node, f);
     break;
   default:
-    fatal_source(file_id, node->pn_pos.begin.lineno, "unsupported node type (%d)", node->pn_type);
+    fatal_source(file_id, node->pn_pos.begin.lineno, "unsupported node type (%u)", (unsigned int) node->pn_type);
   }
 }
 
@@ -911,7 +1037,7 @@ static void output_statement(JSParseNode * node, Stream * f, int indent, bool is
   {
     assert(node->pn_arity == PN_TERNARY);
 
-    uint16_t line = node->pn_pos.begin.lineno;
+    uint32_t line = node->pn_pos.begin.lineno;
     if (! is_jscoverage_if) {
       if (line > num_lines) {
         fatal("file %s contains more than 65,535 lines", file_id);
@@ -926,8 +1052,8 @@ static void output_statement(JSParseNode * node, Stream * f, int indent, bool is
     output_expression(node->pn_kid1, f, false);
     Stream_write_string(f, ") {\n");
     if (is_jscoverage_if && node->pn_kid3) {
-      uint16_t else_start = node->pn_kid3->pn_pos.begin.lineno;
-      uint16_t else_end = node->pn_kid3->pn_pos.end.lineno + 1;
+      uint32_t else_start = node->pn_kid3->pn_pos.begin.lineno;
+      uint32_t else_end = node->pn_kid3->pn_pos.end.lineno + 1;
       Stream_printf(f, "%*s", indent + 2, "");
       Stream_printf(f, "_$jscoverage['%s'].conditionals[%d] = %d;\n", file_id, else_start, else_end);
     }
@@ -940,8 +1066,8 @@ static void output_statement(JSParseNode * node, Stream * f, int indent, bool is
       Stream_write_string(f, "else {\n");
 
       if (is_jscoverage_if) {
-        uint16_t if_start = node->pn_kid2->pn_pos.begin.lineno + 1;
-        uint16_t if_end = node->pn_kid2->pn_pos.end.lineno + 1;
+        uint32_t if_start = node->pn_kid2->pn_pos.begin.lineno + 1;
+        uint32_t if_end = node->pn_kid2->pn_pos.end.lineno + 1;
         Stream_printf(f, "%*s", indent + 2, "");
         Stream_printf(f, "_$jscoverage['%s'].conditionals[%d] = %d;\n", file_id, if_start, if_end);
       }
@@ -1026,7 +1152,7 @@ static void output_statement(JSParseNode * node, Stream * f, int indent, bool is
       assert(node->pn_left->pn_arity == PN_TERNARY);
       Stream_write_string(f, "for (");
       if (node->pn_left->pn_kid1) {
-        output_expression(node->pn_left->pn_kid1, f, false);
+        output_expression(node->pn_left->pn_kid1, f, false, false);
       }
       Stream_write_string(f, ";");
       if (node->pn_left->pn_kid2) {
@@ -1131,7 +1257,7 @@ static void output_statement(JSParseNode * node, Stream * f, int indent, bool is
     assert(node->pn_arity == PN_UNARY);
     Stream_printf(f, "%*s", indent, "");
     if (node->pn_kid != NULL) {
-      output_expression(node->pn_kid, f, true);
+      output_expression(node->pn_kid, f, true, false);
     }
     Stream_write_string(f, ";\n");
     break;
@@ -1227,8 +1353,12 @@ static void output_statement(JSParseNode * node, Stream * f, int indent, bool is
       instrument_statement(p, f, indent, false);
     }
     break;
+  case TOK_NAME:
+    // this is a duplicate function
+    // FIXME
+    break;
   default:
-    fatal_source(file_id, node->pn_pos.begin.lineno, "unsupported node type (%d)", node->pn_type);
+    fatal_source(file_id, node->pn_pos.begin.lineno, "unsupported node type (%u)", (unsigned int) node->pn_type);
   }
 }
 
@@ -1239,7 +1369,7 @@ TOK_EXPORT, TOK_IMPORT are not handled.
 */
 static void instrument_statement(JSParseNode * node, Stream * f, int indent, bool is_jscoverage_if) {
   if (node->pn_type != TOK_LC && node->pn_type != TOK_LEXICALSCOPE) {
-    uint16_t line = node->pn_pos.begin.lineno;
+    uint32_t line = node->pn_pos.begin.lineno;
     if (line > num_lines) {
       fatal("file %s contains more than 65,535 lines", file_id);
     }
@@ -1296,12 +1426,12 @@ void jscoverage_instrument_js(const char * id, const uint16_t * characters, size
   file_id = id;
 
   /* parse the javascript */
-  JSParseContext parse_context;
-  if (! js_InitParseContext(context, &parse_context, NULL, NULL, characters, num_characters, NULL, NULL, 1)) {
+  JSCompiler compiler(context);
+  if (! compiler.init(characters, num_characters, NULL, id, 1)) {
     fatal("cannot create token stream from file %s", file_id);
   }
   JSErrorReporter old_error_reporter = JS_SetErrorReporter(context, error_reporter);
-  JSParseNode * node = js_ParseScript(context, global, &parse_context);
+  JSParseNode * node = compiler.parse(global);
   if (node == NULL) {
     js_ReportUncaughtException(context);
     fatal("parse error in file %s", file_id);
@@ -1324,7 +1454,7 @@ void jscoverage_instrument_js(const char * id, const uint16_t * characters, size
   size_t line_number = 0;
   size_t i = 0;
   while (i < num_characters) {
-    if (line_number == UINT16_MAX) {
+    if (line_number == UINT32_MAX) {
       fatal("file %s contains more than 65,535 lines", file_id);
     }
     line_number++;
@@ -1388,27 +1518,54 @@ void jscoverage_instrument_js(const char * id, const uint16_t * characters, size
 
   Stream * instrumented = Stream_new(0);
   instrument_statement(node, instrumented, 0, false);
-  js_FinishParseContext(context, &parse_context);
 
   /* write line number info to the output */
-  Stream_write_string(output, "/* automatically generated by JSCoverage - do not edit */\n");
-  if (jscoverage_mozilla) {
+  Stream_write_string(output, JSCOVERAGE_INSTRUMENTED_HEADER);
+  switch (jscoverage_mode) {
+  case JSCOVERAGE_MOZILLA:
     Stream_write_string(output, "try {\n");
-    Stream_write_string(output, "  Components.utils.import('resource://gre/modules/jscoverage.jsm');\n");
+    Stream_write_string(output, "  Components.utils.import('resource://app/modules/jscoverage.jsm');\n");
     Stream_printf(output, "  dump('%s: successfully imported jscoverage module\\n');\n", id);
     Stream_write_string(output, "}\n");
     Stream_write_string(output, "catch (e) {\n");
     Stream_write_string(output, "  _$jscoverage = {};\n");
     Stream_printf(output, "  dump('%s: failed to import jscoverage module - coverage not available for this file\\n');\n", id);
     Stream_write_string(output, "}\n");
-  }
-  else {
-    Stream_write_string(output, "if (! top._$jscoverage) {\n  top._$jscoverage = {};\n}\n");
-    Stream_write_string(output, "var _$jscoverage = top._$jscoverage;\n");
+    break;
+  case JSCOVERAGE_NORMAL:
+    /*
+    // pseudo-code:
+    if (top && top.opener && top.opener._$jscoverage) {
+      var _$jscoverage = top._$jscoverage = top.opener._$jscoverage;
+    }
+    else if (top && top.opener) {
+      var _$jscoverage = top._$jscoverage = top.opener._$jscoverage = {};
+    }
+    else if (top && top._$jscoverage) {
+      var _$jscoverage = top._$jscoverage;
+    }
+    else if (top) {
+      var _$jscoverage = top._$jscoverage = {};
+    }
+    else if (_$jscoverage) {
+      // nothing to do!
+    }
+    else {
+      var _$jscoverage = {};
+    }
+    */
+    {
+      const struct Resource * resource = get_resource("header.js");
+      Stream_write(output, resource->data, resource->length);
+    }
+    break;
+  case JSCOVERAGE_NO_BROWSER:
+    Stream_write_string(output, "if (typeof _$jscoverage === 'undefined') {\n  var _$jscoverage = {};\n}\n");
+    break;
   }
   Stream_printf(output, "if (! _$jscoverage['%s']) {\n", file_id);
   Stream_printf(output, "  _$jscoverage['%s'] = [];\n", file_id);
-  for (int i = 0; i < num_lines; i++) {
+  for (uint32_t i = 0; i < num_lines; i++) {
     if (lines[i]) {
       Stream_printf(output, "  _$jscoverage['%s'][%d] = 0;\n", file_id, i + 1);
     }
@@ -1419,6 +1576,11 @@ void jscoverage_instrument_js(const char * id, const uint16_t * characters, size
   free(exclusive_directives);
   exclusive_directives = NULL;
 
+  /* copy the original source to the output */
+  Stream_printf(output, "_$jscoverage['%s'].source = ", file_id);
+  jscoverage_write_source(id, characters, num_characters, output);
+  Stream_printf(output, ";\n");
+
   /* conditionals */
   if (has_conditionals) {
     Stream_printf(output, "_$jscoverage['%s'].conditionals = [];\n", file_id);
@@ -1443,11 +1605,6 @@ void jscoverage_instrument_js(const char * id, const uint16_t * characters, size
     free(if_directive);
   }
 
-  /* copy the original source to the output */
-  Stream_printf(output, "_$jscoverage['%s'].source = ", file_id);
-  jscoverage_write_source(id, characters, num_characters, output);
-  Stream_printf(output, ";\n");
-
   Stream_delete(instrumented);
 
   file_id = NULL;
@@ -1687,12 +1844,12 @@ int jscoverage_parse_json(Coverage * coverage, const uint8_t * json, size_t leng
 
   JS_free(context, base);
 
-  JSParseContext parse_context;
-  if (! js_InitParseContext(context, &parse_context, NULL, NULL, parenthesized_json, length + 2, NULL, NULL, 1)) {
+  JSCompiler compiler(context);
+  if (! compiler.init(parenthesized_json, length + 2, NULL, NULL, 1)) {
     free(parenthesized_json);
     return -1;
   }
-  JSParseNode * root = js_ParseScript(context, global, &parse_context);
+  JSParseNode * root = compiler.parse(global);
   free(parenthesized_json);
 
   JSParseNode * semi = NULL;
@@ -1881,6 +2038,5 @@ int jscoverage_parse_json(Coverage * coverage, const uint8_t * json, size_t leng
   }
 
 done:
-  js_FinishParseContext(context, &parse_context);
   return result;
 }
