@@ -71,10 +71,6 @@ static void check_contains_file(const char * file1, const char * file2) {
 }
 
 static void instrument_file(const char * source_file, const char * destination_file, const char * id, int instrumenting) {
-  if (g_verbose) {
-    printf("Instrumenting file %s\n", id);
-  }
-
   /* check if they are the same */
   char * canonical_source_file = make_canonical_path(source_file);
   char * canonical_destination_file = make_canonical_path(destination_file);
@@ -87,10 +83,17 @@ static void instrument_file(const char * source_file, const char * destination_f
     switch (file_type) {
     case FILE_TYPE_OTHER:
     case FILE_TYPE_HTML:
+      if (g_verbose) {
+        printf("Copying file %s\n", id);
+      }
       copy_file(source_file, destination_file);
       break;
     case FILE_TYPE_JS:
       {
+        if (g_verbose) {
+          printf("Instrumenting file %s\n", id);
+        }
+
         FILE * input = xfopen(source_file, "rb");
         FILE * output = xfopen(destination_file, "wb");
 
@@ -133,6 +136,10 @@ static void instrument_file(const char * source_file, const char * destination_f
     }
   }
   else {
+    if (g_verbose) {
+      printf("Copying file %s (on --no-instrument list)\n", id);
+    }
+
     copy_file(source_file, destination_file);
   }
 }
