@@ -42,6 +42,8 @@ static const char * specified_encoding = NULL;
 const char * jscoverage_encoding = "ISO-8859-1";
 bool jscoverage_highlight = true;
 
+struct in_addr jscoverage_ip_address;
+
 typedef struct SourceCache {
   char * url;
   uint16_t * characters;
@@ -696,13 +698,32 @@ static void handle_jscoverage_request(HTTPExchange * exchange) {
       return;
     }
 
-    /* allow only from localhost */
+    /* check client IP address */
     struct sockaddr_in client;
     if (HTTPExchange_get_peer(exchange, &client) != 0) {
       send_response(exchange, 500, "Cannot get client address\n");
       return;
     }
-    if (client.sin_addr.s_addr != htonl(INADDR_LOOPBACK)) {
+
+    /*
+    We allow connections from loopback 127.0.0.0 to 127.255.255.255 (127/8)
+    and from the address specified with --ip-address
+    */
+    bool client_allowed = false;
+
+    if (client.sin_addr.s_addr == htonl(INADDR_LOOPBACK)) {
+      client_allowed = true;
+    }
+
+    if ((ntohl(client.sin_addr.s_addr) & 0xFF000000) == 0x7F000000) {
+      client_allowed = true;
+    }
+
+    if (client.sin_addr.s_addr == jscoverage_ip_address.s_addr) {
+      client_allowed = true;
+    }
+
+    if (! client_allowed) {
       send_response(exchange, 403, "This operation can be performed only by localhost\n");
       return;
     }
@@ -1276,8 +1297,7 @@ int main(int argc, char ** argv) {
     }
 #endif
 
-    /* INADDR_LOOPBACK */
-    HTTPConnection * connection = HTTPConnection_new_client("127.0.0.1", numeric_port);
+    HTTPConnection * connection = HTTPConnection_new_client(ip_address, numeric_port);
     if (connection == NULL) {
       fatal("could not connect to server");
     }
@@ -1319,6 +1339,12 @@ InitializeCriticalSection(&source_cache_mutex);
     printf("Starting HTTP server on %s:%lu\n", ip_address, numeric_port);
     fflush(stdout);
   }
+
+  /* set the IP address */
+  if (! inet_aton(ip_address, &jscoverage_ip_address)) {
+    fatal("invalid IP address: %s\n", ip_address);
+  }
+
   HTTPServer_run(ip_address, (uint16_t) numeric_port, handler);
   if (verbose) {
     printf("Stopping HTTP server\n");
