@@ -29,21 +29,15 @@ cleanup() {
 
 trap 'cleanup' 0 1 2 3 15
 
-export PATH=.:..:$PATH
-
-if [ -z "$VALGRIND" ]
-then
-  delay=0.2
-else
-  delay=2
-fi
+. ./common.sh
 
 $VALGRIND jscoverage-server --proxy > OUT 2> ERR &
 proxy_server_pid=$!
 proxy_server_port=8080
 
-sleep $delay
+wait_for_server http://127.0.0.1:8080/jscoverage.html
 
+unused_port=`perl unused-port.pl`
 echo 504 > EXPECTED
-! curl -f -w '%{http_code}\n' -x 127.0.0.1:8080 http://127.0.0.1:1/index.html 2> /dev/null > ACTUAL
+! curl -f -w '%{http_code}\n' -x 127.0.0.1:8080 http://127.0.0.1:$unused_port/index.html 2> /dev/null > ACTUAL
 diff EXPECTED ACTUAL
