@@ -29,19 +29,12 @@ cleanup() {
 
 trap 'cleanup' 0 1 2 3 15
 
-export PATH=.:..:$PATH
-
-if [ -z "$VALGRIND" ]
-then
-  delay=0.2
-else
-  delay=2
-fi
+. ./common.sh
 
 $VALGRIND jscoverage-server --port 8000 > OUT 2> ERR &
 server_pid=$!
 server_port=8000
 
-sleep $delay
+wait_for_server http://127.0.0.1:8000/jscoverage.html
 
 ./http-client-close-after-request
