@@ -28,14 +28,7 @@ cleanup() {
 
 trap 'cleanup' 0 1 2 3 15
 
-export PATH=.:..:$PATH
-
-if [ -z "$VALGRIND" ]
-then
-  delay=0.2
-else
-  delay=2
-fi
+. ./common.sh
 
 rm -fr DIR
 mkdir DIR
@@ -43,7 +36,7 @@ $VALGRIND jscoverage-server --document-root=DIR &
 server_pid=$!
 server_port=8080
 
-sleep $delay
+wait_for_server http://127.0.0.1:8080/jscoverage.html
 
 # make some funny files
 echo x > 'DIR/1&2.txt'
